from timemachines.skaters.sk.sktheta import SK_THETA_SKATERS
from timemachines.skaters.sk.skautoarima import SK_AA_SKATERS
from timemachines.skaters.sk.skautoets import SK_AE_SKATERS
SK_SKATERS = SK_THETA_SKATERS + SK_AA_SKATERS + SK_AE_SKATERS
