from timemachines.skaters.proph.prophskaterssingular import PROPHET_SKATERS_SINGULAR, PROPHET_R2_SKATERS
from timemachines.skaters.proph.prophskaterscomposed import PROPHET_SKATERS_COMPOSED


PROPHET_SKATERS = PROPHET_SKATERS_SINGULAR + PROPHET_SKATERS_COMPOSED


